#!/usr/bin/env python3
#
# File: format_data.py
#
# Created: Thursday, April  2 2015 by rejuvyesh <mail@rejuvyesh.com>
# License: GNU GPL 3 <http://www.gnu.org/copyleft/gpl.html>
#

import csv
import glob
import os


def read_words(files):
  """
  
  Arguments:
  - `files`:
  """
  words = []
  for filename in files:
    with open(filename) as f:
      txt = f.read().strip().split()
      words.append(txt)
  return words


def read_polarity(loc):
  """ Read Polarity Data

  Arguments:
  - `loc`:
  """
  pos = read_words(glob.iglob(os.path.join(loc, 'pos',  '*.txt')))
  neg = read_words(glob.iglob(os.path.join(loc, 'neg',  '*.txt')))
  return pos, neg


def read_review(loc):
  # Positive
  with open(os.path.join(loc, 'Positive.txt')) as fp:
    plines = fp.read().strip().splitlines()
  pos = [x.split() for x in plines]

  # Negative
  with open(os.path.join(loc, 'Negative.txt')) as fn:
    nlines = fn.read().strip().splitlines()
  neg = [x.split() for x in nlines]

  return pos, neg


def main():
  loc = 'hindi_data/data/'
  tourism_data_loc = loc + 'Hindi_Sense_Tagged_Data_Tourism/'
  product_loc = loc + 'ProductReviews (350+350)'
  polar_loc = loc + 'HindiPolarityLabeledCorpora'
  jagaran_loc = loc + 'Jagaran'
  posp, negp = read_polarity(polar_loc)
  # post, negt = read_tourism(tourism_data_loc)
  posr, negr = read_review(product_loc)
  posj, negj = read_review(jagaran_loc)
  pos = posp + posr + posj
  neg = negp + negr + negj
  with open('Hindi_Pos.csv', 'w') as f:
    csv.writer(f, delimiter=',').writerows(pos)
  with open('Hindi_Neg.csv', 'w') as f:
    csv.writer(f, delimiter=',').writerows(neg)

  pos_set = set(open('Hindi_Pos.csv').read().strip().split(','))
  neg_set = set(open('Hindi_Neg.csv').read().strip().split(','))
  vocab = pos_set.union(neg_set)
  with open('Hindi_vocab.txt', 'w') as vf:
    vf.write('\n'.join(vocab))
  with open('Hindi_vocab.size', 'w') as sf:
    sf.write('{}'.format(len(vocab)))

if __name__ == '__main__':
  main()
