function gen_data(filename, embed_size)
          

% Main function to run to generate data!
%
% Author: Jayesh Kumar Gupta, 2014.
%
% Contact: Jayesh Kumar Gupta http://rejuvyesh.com
%          Indian Institute of Technology, Kanpur, India


% Vocab
vocab =  textscan(fopen('Hindi_vocab.txt'), '%s', 'Delimiter', '\n');
vocab_id = vocab{1};
vocab_size = length(vocab_id);

% Read Files
posdat = csvread('Hindi_Pos_index.csv');
negdat = csvread('Hindi_Neg_index.csv');

% Truncate columns
negdat = negdat(:,1:size(posdat, 2));

% Join matrices
dat = [posdat; negdat];

% Labels
labels = ones(length(posdat)+length(negdat),2);
labels(length(posdat)+1:end,1) = labels(length(posdat)+1:end,1)*2;

findInd = @(r) norm(find(r>vocab_size, 1, 'first'));
temp = num2cell(dat,2);
labels(:,2) = cellfun(findInd, temp);

% Shuffle the data & labels
shuff_ind = randperm(size(labels,1));
shuf_labels = labels(shuff_ind,:);
shuf_data = dat(shuff_ind,:);

% train test data indices
cv_ind = crossvalind('Kfold', size(shuf_data, 1), 5);
test_ind = (cv_ind == 1); valid_ind = (cv_ind == 2); 
train_ind = ~(test_ind | valid_ind);
% train test data
train_data = shuf_data(train_ind,:);
train_label = shuf_labels(train_ind,:);
valid_data = shuf_data(valid_ind,:);
valid_label = shuf_labels(valid_ind,:);
test_data = shuf_data(test_ind,:);
test_label = shuf_labels(test_ind,:);


% rename data
sent_length = size(shuf_data, 2);
size_vocab = vocab_size+1;
index = vocab_id;
test = test_data;
test_lbl = test_label;
valid = valid_data;
valid_lbl = valid_label;
train = train_data;
train_lbl = train_label;

% random generate embeddings
vocab_emb = randn(embed_size, vocab_size);

% Need to get vocab_id i.e. word embeddings for the vocabulary

clearvars -except sent_length size_vocab index test train valid test_lbl ...
    valid_lbl train_lbl vocab_emb filename;

save(filename)
end
